/*
 * This program tests the operation of the SPI driver for the AVR.
 * @author Akop Palyan
 */
program TestSPI {

   components { TestSPI, Mica2, MCU, SPI }

   entrypoint main = TestSPI.entry;
   entrypoint spi_stc = SPI.interruptHandler;
   //entrypoint timer0_comp = Timer0.compareHandler;
}

component TestSPI {

    field master: boolean = true;
    field buffer: char[] = new char[32];
    field windex: int = 0;
    field rindex: int = 0;

    method entry() {
	Mica2.startSPI(master);
	MCU.enable();
	receive();
	transmit();
    }

    method receive() {
	if ( master ) receiveMaster();
	else receiveSlave();
    }

    method receiveMaster() {
	// in master mode, we must transfer data in order to receive data.
	while (windex < buffer.length) {
	    windex = windex + SPI.write(buffer, windex, buffer.length - windex);
	    while (rindex < windex) {
	        rindex = rindex + SPI.read(buffer, rindex, windex - rindex);
	    }
	}
    }

    method receiveSlave() {
        while (rindex < buffer.length) {
	    rindex = rindex + SPI.read(buffer, rindex, buffer.length - rindex);
	}
    }

    method transmit() {
	while (windex < buffer.length) {
	    windex = windex + SPI.write(buffer, windex, buffer.length - windex);
	}
    }
}
