component MCU {
    field OCF0: int = 1; // bit number for output compare flag

    method sleep() {
	// TODO: implement actual sleeping of MCU.
	device.SREG = (0b10000000); // enable interrupts
        device.MCUCR = 0x00;
    }

    method disable(): 8 {
	local mask = device.SREG;
	device.SREG[7] = 0b0;
	return mask;
    }

    method restore(mask: 8) {
	device.SREG = mask;
    }

    method enable() {
	device.SREG = 0b10000000;
    }

    method wait(time: int) { // time in milliseconds
       while (time > 0) {
          wait_ms();
	  time--;
       }
    }

    method wait_ms() {
       local counter: int;
       local mask: 8;
       local divider: 3;

       mask = MCU.disable();		// disable interrupts
       counter = Timer0.getCount();	// save current count
       divider = Timer0.divider;	// save current divider

       device.TIFR[OCF0] = 0b1; // clear compare flag
       Timer0.setCount(device.OCR0 :: int - 1);
       if (divider != Timer0.DIV_32)
          Timer0.setDivider(Timer0.DIV_32);

       while (device.TIFR[OCF0] != 0b1) // wait until match occurs
       { }

       Timer0.setCount(counter);	// restore current count
       if (divider != Timer0.DIV_32)
          Timer0.setDivider(divider);	// restore current divider
       MCU.restore(mask);		// enable interrupts
    }
}

